import sys
import os

# решается проблема запуска не из модуля
PACKAGE_PARENT = '..'
SCRIPT_DIR = os.path.dirname(os.path.realpath(os.path.join(os.getcwd(), os.path.expanduser(__file__))))
sys.path.append(os.path.normpath(os.path.join(SCRIPT_DIR, PACKAGE_PARENT)))

from autochests_bot.start_bot import start_bot  # noqa

start_bot()



"""Файл содержит аргументы для управления запуском бота."""
import argparse

parser = argparse.ArgumentParser(description='Запуск бота для автоматического сбора сундуков на twitch.')
parser.add_argument(
    '--debug',
    default=False,
    help='При установке этого ключа, скрипт будет запущен в дебаг режиме.',
    action='store_true'
)

parser.add_argument(
    '--screen',
    default=(1590, 1010, 1660, 1070),
    type=int,
    nargs='+',
    help='Координаты внутри которых будет обрабатываться изображение. Передавать через пробел x1 y1 x2 y2'
)

parser.add_argument(
    '--chest',
    default=(1617, 1048),
    type=int,
    nargs='+',
    help='Координаты куда будет устанавливаться курсор для клика. Передавать через пробел x y'
)

parser.add_argument(
    '--color',
    default=(0, 230, 203),
    type=int,
    nargs='+',
    help='Искомый цвет на обрабатываемом участке экрана. Передавать в формате RGB через пробел'
)

parser.add_argument(
    '--timer',
    default=0,
    type=int,
    help='Время, на которое будет запущен бот'
)

namespace = parser.parse_args()



"""Файл с основным классом бота, отвечающим за работу бота."""
import time

import numpy as np
import pyscreenshot as sc
from pynput.mouse import Button as MButton, Controller as MController


class Bot:
    def __init__(self, screen, chest, color):
        self.SCREEN = screen  # Место экрана которое будет обрабатываться
        self.CHEST = chest  # Место где находится сундук
        self.COLOR_FOR_FIND = np.array(color)  # цвет который будет искаться на месте обработки
        self.MOUSE_CONTROLLER = MController()

    def bot_execute(self):
        counter = 0
        im = sc.grab(bbox=self.SCREEN)
        array = np.array(im)
        for i in range(array.shape[0]):
            for j in range(array.shape[1]):
                # если искомый пиксель есть в обрабатываемой области экрана, увеличиваем счетчик
                if np.array_equal(array[i][j], self.COLOR_FOR_FIND):
                    counter += 1
        # если в обрабатываемой области достаточно искомых пикселей, перемещаем курсор в нужное места экрана
        # и производим клин
        if counter > 1000:
            mouse_position = self.MOUSE_CONTROLLER.position
            self.MOUSE_CONTROLLER.position = self.CHEST
            self.MOUSE_CONTROLLER.click(MButton.left, 1)
            time.sleep(0.1)
            self.MOUSE_CONTROLLER.position = mouse_position

    def debug(self):
        """
        Метод вызываемый для отладки.

        Производит отображения области экрана которая будет обрабатываться, место куда будет установлен курсор
        и количество пикселей того или иного цвета.
        """
        time.sleep(1)
        im = sc.grab(bbox=self.SCREEN)
        im.show()

        array = np.array(im)
        unique = dict()
        for i in range(array.shape[0]):
            for j in range(array.shape[1]):
                # ToDo when running on a monotonous surface (processing an area of the screen of the same color),
                #  it throws an error
                """Traceback (most recent call last):
                File "/usr/lib64/python3.8/runpy.py", line 193, in _run_module_as_main
                return _run_code(code, main_globals, None,
                File "/usr/lib64/python3.8/runpy.py", line 86, in _run_code
                exec(code, run_globals)
                File "/home/andrey/projects/tes/env/lib/python3.8/site-packages/autochests_bot/__main__.py", line 11, in <module>
                start_bot()
                File "/home/andrey/projects/tes/env/lib/python3.8/site-packages/autochests_bot/start_bot.py", line 15, in start_bot
                bot.debug()
                File "/home/andrey/projects/tes/env/lib/python3.8/site-packages/autochests_bot/bot_controll/chests_bot.py", line 50, in debug
                key = f"{array[i][j][0]},{array[i][j][1]},{array[i][j][2]}"
                IndexError: invalid index to scalar variable"""
                # Todo array print
                """
                [[0 1 1 ... 5 5 5]
                [1 1 1 ... 5 5 5]
                [4 1 1 ... 5 5 5]
                ...
                [5 5 5 ... 5 5 5]
                [5 5 5 ... 5 5 5]
                [5 5 5 ... 5 5 5]]
                """
                # ToDo write error handling
                key = f"{array[i][j][0]},{array[i][j][1]},{array[i][j][2]}"
                if unique.get(key, None):
                    unique[key] += 1
                else:
                    unique[key] = 1

        sorted_unique = {k: v for k, v in sorted(unique.items(), key=lambda item: item[1])}
        print(sorted_unique)

        position = self.MOUSE_CONTROLLER.position
        self.MOUSE_CONTROLLER.position = self.CHEST
        time.sleep(5)
        self.MOUSE_CONTROLLER.position = position



"""Файл для управления запуском ботом."""
import time
import os
import sys
from signal import SIGTERM

from .bot_controll.chests_bot import Bot
from .bot_controll.args import namespace

bot = Bot(namespace.screen, namespace.chest, namespace.color)


def start_bot():
    if namespace.debug:
        bot.debug()
    else:
        if namespace.timer:
            # ToDo не проверил как работает для винды, возможно нужно будет переписать с вилки на мультипроцессы
            pid = os.fork()
            if pid == 0:
                # запускаем бота в дочке
                while True:
                    time.sleep(60)
                    bot.bot_execute()
            if pid > 0:
                # основой процесс слипаем на время указанное в таймере
                time.sleep(namespace.timer)
                # убийство дочернего процесса
                try:
                    os.kill(pid, SIGTERM)
                except OSError as e:
                    if "No such process" not in e.strerror or "Нет такого процесса" not in e.strerror:
                        sys.exit(1)
                    else:
                        print(e.strerror)
                        sys.exit(1)
        else:
            while True:
                time.sleep(60)
                bot.bot_execute()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="twitch-chest-bot",
    version="0.9.3",
    author="J4CK VVH173",
    author_email="author@example.com",
    description="Twitch bot for auto collection of chests",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/J4CKVVH173/twitch_auto_chests",
    packages=setuptools.find_packages(),
    install_requires=[
            'daemon-process',
            'numpy==1.18.1',
            'pynput==1.6.6',
            'pyscreenshot==0.6',
            'Pillow==7.0.0',
        ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Unix",
    ],
    python_requires='>=3.6',
)
